"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MotifAvatar = void 0;
const tslib_1 = require("tslib");
const Decimal_1 = require("./Decimal");
const abstract_signer_1 = require("@ethersproject/abstract-signer");
const typechain_1 = require("@motif-foundation/asset/dist/typechain");
const addresses_1 = require("./addresses");
const utils_1 = require("./utils");
class MotifAvatar {
    constructor(signerOrProvider, chainId) {
        if (abstract_signer_1.Signer.isSigner(signerOrProvider)) {
            this.readOnly = false;
        }
        else {
            this.readOnly = true;
        }
        this.signerOrProvider = signerOrProvider;
        this.chainId = chainId;
        const network = (0, utils_1.chainIdToNetworkName)(chainId);
        this.avatarAddress = addresses_1.addresses[network].avatar;
        this.avatarExchangeAddress = addresses_1.addresses[network].avatarExchange;
        this.avatar = typechain_1.AvatarFactory.connect(this.avatarAddress, signerOrProvider);
        this.avatarExchange = typechain_1.AvatarExchangeFactory.connect(this.avatarExchangeAddress, signerOrProvider);
    }
    /*********************
     * Motif View Methods
     *********************
     */
    /**
     * Fetches the content hash for the specified avatar on the Motif Avatar Contract
     * @param avatarId
     */
    fetchContentHash(avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatar.tokenContentHashes(avatarId);
        });
    }
    /**
     * Fetches the metadata hash for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchMetadataHash(avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatar.tokenMetadataHashes(avatarId);
        });
    }
    /**
     * Fetches the metadata hash for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchTokenContract(avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatar.tokenContractAddresses(avatarId);
        });
    }
    /**
     * Fetches the content uri for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchContentURI(avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatar.tokenURI(avatarId);
        });
    }
    /**
     * Fetches the metadata uri for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchMetadataURI(avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatar.tokenMetadataURI(avatarId);
        });
    }
    /**
     * Fetches the creator for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchCreator(avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatar.tokenCreators(avatarId);
        });
    }
    /**
     * Fetches the current bid shares for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchCurrentBidShares(avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatarExchange.bidSharesForToken(avatarId);
        });
    }
    /**
     * Fetches the current ask for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchCurrentAsk(avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatarExchange.currentAskForToken(avatarId);
        });
    }
    /**
     * Fetches the current bid for the specified bidder for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     * @param bidder
     */
    fetchCurrentBidForBidder(avatarId, bidder) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatarExchange.bidForTokenBidder(avatarId, bidder);
        });
    }
    /**
     * Fetches the permit nonce on the specified avatar id for the owner address
     * @param address
     * @param avatarId
     */
    fetchPermitNonce(address, avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatar.permitNonces(address, avatarId);
        });
    }
    /**
     * Fetches the current mintWithSig nonce for the specified address
     * @param address
     * @param avatarId
     */
    fetchMintWithSigNonce(address) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatar.mintWithSigNonces(address);
        });
    }
    /*********************
     * Motif Write Methods
     *********************
     */
    /**
     * Updates the content uri for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     * @param tokenURI
     */
    updateContentURI(avatarId, tokenURI) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
                (0, utils_1.validateURI)(tokenURI);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.avatar.updateTokenURI(avatarId, tokenURI);
        });
    }
    /**
     * Updates the metadata uri for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     * @param metadataURI
     */
    updateMetadataURI(avatarId, metadataURI) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
                (0, utils_1.validateURI)(metadataURI);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.avatar.updateTokenMetadataURI(avatarId, metadataURI);
        });
    }
    /**
     * Mints a new piece of avatar on an instance of the Motif Avatar Contract
     * @param mintData
     * @param bidShares
     */
    mint(avatarData, bidShares) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
                (0, utils_1.validateURI)(avatarData.metadataURI);
                (0, utils_1.validateURI)(avatarData.tokenURI);
                (0, utils_1.validateBidShares)(bidShares.creator, bidShares.owner, bidShares.prevOwner);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            const gasEstimate = yield this.avatar.estimateGas.mint(avatarData, bidShares);
            const paddedEstimate = gasEstimate.mul(110).div(100);
            return this.avatar.mint(avatarData, bidShares, { gasLimit: paddedEstimate.toString() });
        });
    }
    /**
     * Mints a new piece of avatar on an instance of the Motif Avatar Contract
     * @param creator
     * @param avatarData
     * @param bidShares
     * @param sig
     */
    mintWithSig(creator, avatarData, bidShares, sig) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
                (0, utils_1.validateURI)(avatarData.metadataURI);
                (0, utils_1.validateURI)(avatarData.tokenURI);
                (0, utils_1.validateBidShares)(bidShares.creator, bidShares.owner, bidShares.prevOwner);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.avatar.mintWithSig(creator, avatarData, bidShares, sig);
        });
    }
    /**
     * Sets an ask on the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     * @param ask
     */
    setAsk(avatarId, ask) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.avatar.setAsk(avatarId, ask);
        });
    }
    /**
     * Sets a bid on the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     * @param bid
     */
    setBid(avatarId, bid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.avatar.setBid(avatarId, bid);
        });
    }
    /**
     * Removes the ask on the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    removeAsk(avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.avatar.removeAsk(avatarId);
        });
    }
    /**
     * Removes the bid for the msg.sender on the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    removeBid(avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.avatar.removeBid(avatarId);
        });
    }
    /**
     * Accepts the specified bid on the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     * @param bid
     */
    acceptBid(avatarId, bid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.avatar.acceptBid(avatarId, bid);
        });
    }
    /**
     * Grants the spender approval for the specified avatar using meta transactions as outlined in EIP-712
     * @param sender
     * @param avatarId
     * @param sig
     */
    permit(spender, avatarId, sig) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.avatar.permit(spender, avatarId, sig);
        });
    }
    /**
     * Revokes the approval of an approved account for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    revokeApproval(avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.avatar.revokeApproval(avatarId);
        });
    }
    /**
     * Burns the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    burn(avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.avatar.burn(avatarId);
        });
    }
    /***********************
     * ERC-721 View Methods
     ***********************
     */
    /**
     * Fetches the total balance of avatar owned by the specified owner on an instance of the Motif Avatar Contract
     * @param owner
     */
    fetchBalanceOf(owner) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatar.balanceOf(owner);
        });
    }
    /**
     * Fetches the owner of the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchOwnerOf(avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatar.ownerOf(avatarId);
        });
    }
    /**
     * Fetches the avatarId of the specified owner by index on an instance of the Motif Avatar Contract
     * @param owner
     * @param index
     */
    fetchAvatarOfOwnerByIndex(owner, index) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatar.tokenOfOwnerByIndex(owner, index);
        });
    }
    /**
     * Fetches the total amount of non-burned avatar that has been minted on an instance of the Motif Avatar Contract
     */
    fetchTotalAvatar() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatar.totalSupply();
        });
    }
    /**
     * Fetches the avatarId by index on an instance of the Motif Avatar Contract
     * @param index
     */
    fetchAvatarByIndex(index) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatar.tokenByIndex(index);
        });
    }
    /**
     * Fetches the approved account for the specified avatar on an instance of the Motif Avatar Contract
     * @param avatarId
     */
    fetchApproved(avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatar.getApproved(avatarId);
        });
    }
    /**
     * Fetches if the specified operator is approved for all avatar owned by the specified owner on an instance of the Motif Avatar Contract
     * @param owner
     * @param operator
     */
    fetchIsApprovedForAll(owner, operator) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.avatar.isApprovedForAll(owner, operator);
        });
    }
    /***********************
     * ERC-721 Write Methods
     ***********************
     */
    /**
     * Grants approval to the specified address for the specified avatar on an instance of the Motif Avatar Contract
     * @param to
     * @param avatarId
     */
    approve(to, avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.avatar.approve(to, avatarId);
        });
    }
    /**
     * Grants approval for all avatar owner by msg.sender on an instance of the Motif Avatar Contract
     * @param operator
     * @param approved
     */
    setApprovalForAll(operator, approved) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.avatar.setApprovalForAll(operator, approved);
        });
    }
    /**
     * Transfers the specified avatar to the specified to address on an instance of the Motif Avatar Contract
     * @param from
     * @param to
     * @param avatarId
     */
    transferFrom(from, to, avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.avatar.transferFrom(from, to, avatarId);
        });
    }
    /**
     * Executes a SafeTransfer of the specified avatar to the specified address if and only if it adheres to the ERC721-Receiver Interface
     * @param from
     * @param to
     * @param avatarId
     */
    safeTransferFrom(from, to, avatarId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.avatar.safeTransferFrom(from, to, avatarId);
        });
    }
    /****************
     * Miscellaneous
     * **************
     */
    /**
     * Returns the EIP-712 Domain for an instance of the Motif Avatar Contract
     */
    eip712Domain() {
        // Due to a bug in ganache-core, set the chainId to 1 if its a local blockchain
        // https://github.com/trufflesuite/ganache-core/issues/515
        const chainId = this.chainId == 50 ? 1 : this.chainId;
        return {
            name: 'Motif',
            version: '1',
            chainId: chainId,
            verifyingContract: this.avatarAddress,
        };
    }
    /**
     * Checks to see if a Bid's amount is evenly splittable given the avatar's current bidShares
     *
     * @param avatarId
     * @param bid
     */
    isValidBid(avatarId, bid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const isAmountValid = yield this.avatarExchange.isValidBid(avatarId, bid.amount);
            const decimal100 = Decimal_1.Decimal.new(100);
            const currentBidShares = yield this.fetchCurrentBidShares(avatarId);
            const isSellOnShareValid = bid.sellOnShare.value.lte(decimal100.value.sub(currentBidShares.creator.value));
            return isAmountValid && isSellOnShareValid;
        });
    }
    /**
     * Checks to see if an Ask's amount is evenly splittable given the avatar's current bidShares
     *
     * @param avatarId
     * @param ask
     */
    isValidAsk(avatarId, ask) {
        return this.avatarExchange.isValidBid(avatarId, ask.amount);
    }
    /**
     * Checks to see if a piece of avatar has verified uris that hash to their immutable hashes
     *
     * @param avatarId
     * @param timeout
     */
    isVerifiedAvatar(avatarId, timeout = 10) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                const [tokenURI, metadataURI, contentHash, metadataHash] = yield Promise.all([
                    this.fetchContentURI(avatarId),
                    this.fetchMetadataURI(avatarId),
                    this.fetchContentHash(avatarId),
                    this.fetchMetadataHash(avatarId),
                ]);
                const avatarData = (0, utils_1.constructAvatarData)(tokenURI, metadataURI, contentHash, metadataHash);
                return (0, utils_1.isAvatarDataVerified)(avatarData, timeout);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
        });
    }
    /******************
     * Private Methods
     ******************
     */
    /**
     * Throws an error if called on a readOnly == true instance of Motif Sdk
     * @private
     */
    ensureNotReadOnly() {
        if (this.readOnly) {
            throw new Error('ensureNotReadOnly: readOnly Motif instance cannot call contract methods that require a signer.');
        }
    }
}
exports.MotifAvatar = MotifAvatar;
//# sourceMappingURL=MotifAvatar.js.map