"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MotifLand = void 0;
const tslib_1 = require("tslib");
const Decimal_1 = require("./Decimal");
const abstract_signer_1 = require("@ethersproject/abstract-signer");
const typechain_1 = require("@motif-foundation/asset/dist/typechain");
const addresses_1 = require("./addresses");
const utils_1 = require("./utils");
const tiny_invariant_1 = (0, tslib_1.__importDefault)(require("tiny-invariant"));
class MotifLand {
    constructor(signerOrProvider, chainId, landAddress, landExchangeAddress, spaceContractAddress, landOperatorAddr) {
        if (!landAddress != !landExchangeAddress) {
            (0, tiny_invariant_1.default)(false, "Motif Constructor: landAddress and landExchangeAddress must both be non-null or both be null");
        }
        if (abstract_signer_1.Signer.isSigner(signerOrProvider)) {
            this.readOnly = false;
        }
        else {
            this.readOnly = true;
        }
        this.signerOrProvider = signerOrProvider;
        this.chainId = chainId;
        if (landAddress && landExchangeAddress) {
            const parsedLandAddress = (0, utils_1.validateAndParseAddress)(landAddress);
            const parsedLandExchangeAddress = (0, utils_1.validateAndParseAddress)(landExchangeAddress);
            this.landAddress = parsedLandAddress;
            this.landExchangeAddress = parsedLandExchangeAddress;
        }
        else {
            const network = (0, utils_1.chainIdToNetworkName)(chainId);
            this.landAddress = addresses_1.addresses[network].land;
            this.landExchangeAddress = addresses_1.addresses[network].landExchange;
        }
        this.land = typechain_1.LandFactory.connect(this.landAddress, signerOrProvider);
        this.landExchange = typechain_1.LandExchangeFactory.connect(this.landExchangeAddress, signerOrProvider);
    }
    /*********************
     * Land Read Methods
     *********************
     */
    fetchContentHash(landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.tokenContentHashes(landId);
        });
    }
    fetchMetadataHash(landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.tokenMetadataHashes(landId);
        });
    }
    fetchContentURI(landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.tokenURI(landId);
        });
    }
    fetchMetadataURI(landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.tokenMetadataURI(landId);
        });
    }
    fetchCreator(landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.tokenCreators(landId);
        });
    }
    fetchXCoordinate(landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.xCoordinate(landId);
        });
    }
    fetchYCoordinate(landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.yCoordinate(landId);
        });
    }
    fetchCurrentBidShares(landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.landExchange.bidSharesForToken(landId);
        });
    }
    fetchCurrentAsk(landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.landExchange.currentAskForToken(landId);
        });
    }
    fetchCurrentBidForBidder(landId, bidder) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.landExchange.bidForTokenBidder(landId, bidder);
        });
    }
    fetchPermitNonce(address, landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.permitNonces(address, landId);
        });
    }
    fetchMintWithSigNonce(address) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.mintWithSigNonces(address);
        });
    }
    /*********************
     *  Land Write Methods
     *********************
     */
    updateContentURI(landId, tokenURI) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
                (0, utils_1.validateURI)(tokenURI);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.land.updateTokenURI(landId, tokenURI);
        });
    }
    updateMetadataURI(landId, metadataURI) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
                (0, utils_1.validateURI)(metadataURI);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.land.updateTokenMetadataURI(landId, metadataURI);
        });
    }
    mint(landData, bidShares) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
                (0, utils_1.validateURI)(landData.metadataURI);
                (0, utils_1.validateURI)(landData.tokenURI);
                (0, utils_1.validateBidShares)(bidShares.creator, bidShares.owner, bidShares.prevOwner);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            const gasEstimate = yield this.land.estimateGas.mint(landData, bidShares);
            const paddedEstimate = gasEstimate.mul(110).div(100);
            return this.land.mint(landData, bidShares, {
                gasLimit: paddedEstimate.toString(),
            });
        });
    }
    setAsk(landId, ask) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.land.setAsk(landId, ask);
        });
    }
    setBid(landId, bid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.land.setBid(landId, bid);
        });
    }
    removeAsk(landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.land.removeAsk(landId);
        });
    }
    removeBid(landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.land.removeBid(landId);
        });
    }
    acceptBid(landId, bid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.land.acceptBid(landId, bid);
        });
    }
    permit(spender, landId, sig) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.land.permit(spender, landId, sig);
        });
    }
    revokeApproval(landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.land.revokeApproval(landId);
        });
    }
    burn(landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.land.burn(landId);
        });
    }
    /***********************
     * ERC-721 Read Methods
     ***********************
     */
    fetchBalanceOf(owner) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.balanceOf(owner);
        });
    }
    fetchOwnerOf(landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.ownerOf(landId);
        });
    }
    fetchLandOfOwnerByIndex(owner, index) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.tokenOfOwnerByIndex(owner, index);
        });
    }
    fetchTotalLand() {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.totalSupply();
        });
    }
    fetchLandByIndex(index) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.tokenByIndex(index);
        });
    }
    fetchApproved(landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.getApproved(landId);
        });
    }
    fetchIsApprovedForAll(owner, operator) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            return this.land.isApprovedForAll(owner, operator);
        });
    }
    /***********************
     * ERC-721 Write Methods
     ***********************
     */
    approve(to, landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.land.approve(to, landId);
        });
    }
    setApprovalForAll(operator, approved) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.land.setApprovalForAll(operator, approved);
        });
    }
    transferFrom(from, to, landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.land.transferFrom(from, to, landId);
        });
    }
    safeTransferFrom(from, to, landId) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                this.ensureNotReadOnly();
            }
            catch (err) {
                return Promise.reject(err.message);
            }
            return this.land.safeTransferFrom(from, to, landId);
        });
    }
    /****************
     * Miscellaneous
     * **************
     */
    eip712Domain() {
        // Due to a bug in ganache-core, set the chainId to 1 if its a local blockchain
        // https://github.com/trufflesuite/ganache-core/issues/515
        const chainId = this.chainId == 50 ? 1 : this.chainId;
        return {
            name: "Motif",
            version: "1",
            chainId: chainId,
            verifyingContract: this.landAddress,
        };
    }
    isValidBid(landId, bid) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            const isAmountValid = yield this.landExchange.isValidBid(landId, bid.amount);
            const decimal100 = Decimal_1.Decimal.new(100);
            const currentBidShares = yield this.fetchCurrentBidShares(landId);
            const isSellOnShareValid = bid.sellOnShare.value.lte(decimal100.value.sub(currentBidShares.creator.value));
            return isAmountValid && isSellOnShareValid;
        });
    }
    isValidAsk(landId, ask) {
        return this.landExchange.isValidBid(landId, ask.amount);
    }
    isVerifiedLand(landId, timeout = 10) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            try {
                const [tokenURI, metadataURI, contentHash, metadataHash, xCoordinate, yCoordinate] = yield Promise.all([
                    this.fetchContentURI(landId),
                    this.fetchMetadataURI(landId),
                    this.fetchContentHash(landId),
                    this.fetchMetadataHash(landId),
                    this.fetchXCoordinate(landId),
                    this.fetchYCoordinate(landId)
                ]);
                const landData = (0, utils_1.constructLandData)(tokenURI, metadataURI, contentHash, metadataHash, xCoordinate, yCoordinate);
                return (0, utils_1.isLandDataVerified)(landData, timeout);
            }
            catch (err) {
                return Promise.reject(err.message);
            }
        });
    }
    /******************
     * Private Methods
     ******************
     */
    ensureNotReadOnly() {
        if (this.readOnly) {
            throw new Error("ensureNotReadOnly: readOnly Motif instance cannot call contract methods that require a signer.");
        }
    }
}
exports.MotifLand = MotifLand;
//# sourceMappingURL=MotifLand.js.map